SystemOrganization addCategory: #TusLibros!

Object subclass: #Cart
	instanceVariableNames: 'products catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 20:31'!
with: aCatalog
	^self new initializeWith: aCatalog ! !

!Cart methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 20:55'!
add: aProduct 
	(catalog includes: aProduct) ifFalse: [ Error signal: 'Product is not in catalog' ].
	products add: aProduct.
	
! !

!Cart methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 21:09'!
contains: aProduct
	^ products includes: aProduct.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 20:21'!
initialize
	products := Bag new.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 20:25'!
initializeWith: aCatalog 
	products := Bag new.
	catalog := aCatalog! !

!Cart methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 20:21'!
isEmpty
	^ products isEmpty.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 20:30'!
productCount
	^ products size.! !

Object subclass: #Cashier
	instanceVariableNames: 'salesBook'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 17:54'!
emptyCartCheckoutErrorMessage
	^ 'Cart is empty'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 18:17'!
withSalesBook: aSalesBook 
	^self new initializeWithSalesBook: aSalesBook.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 18:35'!
checkout: aCart using: aCreditCard
	| sale |
	
	(aCart isEmpty) ifTrue: [ Error signal: (self class emptyCartCheckoutErrorMessage) ].
	
	sale := Sale withProducts: (aCart products).
	salesBook add: sale.
	
	^ sale! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 18:18'!
initializeWithSalesBook: aSalesBook 
	salesBook := aSalesBook.! !

Object subclass: #Sale
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 20:17'!
aBook
	^	Object new! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 18:27'!
aCartWithCatalog: aCatalog  
	^Cart with: aCatalog! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 17:46'!
aCashier
	^Cashier new! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 20:36'!
aCatalog
	^Set new! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 18:31'!
aCatalogWithProduct: aProduct 
	| catalog |
	
	catalog := Set new.
	catalog add: aProduct.
	
	^catalog! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 18:00'!
aCreditCard
	^'Visa'! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 20:34'!
anEmptyCart
	^	Cart with: (self aCatalog)! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 5/29/2013 20:50'!
anEmptyCatalog
	^ Set new! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 18:16'!
anEmptySalesBook
	^OrderedCollection new! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'FacundoPippia 6/3/2013 18:22'!
testAddProductShouldFailIfProductIsNotInCatalog
	| cart |
	
	cart := self aCartWithCatalog: self anEmptyCatalog.
	
	self should: [cart add: self aBook] 
		raise: Error 
		withExceptionDo: [ :anError |
			self assert: anError messageText = 'Product is not in catalog'.
			self assert: cart isEmpty ]! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'FacundoPippia 5/29/2013 20:18'!
testCartShouldBeEmptyAfterCreation

	self assert: self anEmptyCart isEmpty

 ! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'FacundoPippia 6/3/2013 18:22'!
testCartShouldContainAProductThatWasAdded

	| aCart book|
	book := self aBook.
	aCart := self aCartWithCatalog: (Set with: book).
	aCart add: book.

	self assert: (aCart contains: book).! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'FacundoPippia 6/3/2013 18:22'!
testCartShouldContainTheAmountOfBooksAdded
	"comment stating purpose of message"

	| cart book anotherBook catalog|
	book := self aBook.
	anotherBook := self aBook.
	catalog := (Set with: book with: anotherBook).
	cart := self aCartWithCatalog: catalog.
	cart add: book.
	cart add: anotherBook.
	self assert: (cart productCount = 2).! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'FacundoPippia 6/3/2013 18:22'!
testCartShouldNotBeEmptyAfterAddingABook
	"comment stating purpose of message"
	|cart book catalog|
	book := self aBook.
	catalog := (Set with: book).
	cart := self aCartWithCatalog: (catalog).
	cart add: book.
	self deny: (cart isEmpty).! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 17:59'!
testCheckoutShouldFailIfCartIsEmpty
	| cart cashier salesBook creditCard |
	
	cashier := self aCashier.
	cart := self anEmptyCart.
	salesBook := self anEmptySalesBook.
	creditCard := self aCreditCard.
	
	self should: [ cashier checkout: cart using: creditCard ] 
		raise: Error 
		withExceptionDo: [ :anError | 
			self assert: salesBook isEmpty.
			self assert: anError messageText = Cashier emptyCartCheckoutErrorMessage.
		]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'FacundoPippia 6/3/2013 18:27'!
testSaleIsCreatedWhenCartIsNotEmpty
	| book cart salesBook cashier sale |
	
	book := self aBook.
	
	cart := self aCartWithCatalog: (self aCatalogWithProduct: book).
	cart add: book.
	
	salesBook := self anEmptySalesBook.
	
	cashier := Cashier withSalesBook: salesBook.
		
	sale := cashier checkout: cart using: self aCreditCard.
	
	self assert: (salesBook includes: sale).! !
